#!/bin/env python3
# -*- coding: UTF-8 -*-
# Copyright (c) 2020 growingio.com, Inc.  All Rights Reserved

import ftplib
import json
import csv
import time

from importers.common.common_util import time_format
from importers.common.config_util import FTPConfig, BaseConfig, ApiConfig

from collections import Counter

from importers.common.log_util import logger, my_logger
from importers.data_import.data_model import DataEvent


def check_sv_header_col_value(attr_header_list):
    """
       校验CSV/TSV格式数据头-固定列数的值
    """
    error_list = []

    required_cols = ['event', 'timestamp']

    for col in required_cols:
        if col not in attr_header_list:
            error_list.append(col)

    return error_list


def check_sv_header_col_count(attr_header_list, data_header_list):
    """
       校验CSV/TSV格式数据头-列数
    """
    if len(attr_header_list) != len(data_header_list):
        return False


def check_sv_header_col_order(attr_header_list, data_header_list):
    """
       校验CSV/TSV格式数据头-不为''的顺序
    """
    try:
        for i in range(len(attr_header_list)):
            if attr_header_list[i] != '' and attr_header_list[i] != data_header_list[i]:
                return False
    except Exception:
        return False


def check_sv_col_duplicate(data_list):
    """
       校验CSV/TSV格式数据列名是否重复
    """
    for item in Counter([i for i in data_list if i != '']).items():
        if item[1] > 1:
            return item[0]


def check_csv_header_col_count(cols, line, separator):
    # 创建一个CSV reader
    csv_reader = csv.reader([line], delimiter=separator, quotechar='"')
    columns = next(csv_reader)
    if len(columns) != len(cols):
        return False

    for col_index, col_value in enumerate(columns):
        try:
            json.loads(col_value)
            return True  # 如果成功解析JSON数据，则返回True
        except json.JSONDecodeError:
            continue

    return False


def add_header_to_file(file_path, header_columns, separator):
    """
    向文件添加表头。

    :param file_path: 要修改的文件路径。
    :param header_columns: 表头列名的列表。
    :param separator: 列分隔符。
    """
    with open(file_path, 'r', encoding='utf8') as file:
        original_content = file.read()

    with open(file_path, 'w', encoding='utf8') as file:
        header = separator.join(header_columns) + '\n'
        file.write(header + original_content)


def remove_first_line_from_csv(original_file, output_file=None):
    """
    删除向文件添加的表头。
    """
    if output_file is None:
        output_file = original_file

    with open(original_file, 'r', encoding='utf-8') as file:
        lines = file.readlines()

    with open(output_file, 'w', encoding='utf-8') as file:
        file.writelines(lines[1:])


def replace_first_line_with_header(file_path, header_columns, separator):
    """
    替换文件的第一行为新的表头。

    :param file_path: 要修改的文件路径。
    :param header_columns: 新表头列名的列表。
    :param separator: 列分隔符。
    """
    with open(file_path, 'r', encoding='utf-8') as file:
        lines = file.readlines()

    # 生成新的表头
    header = separator.join(header_columns) + '\n'
    # 替换第一行
    lines[0] = header
    # 将修改后的内容写回文件
    with open(file_path, 'w', encoding='utf-8') as file:
        file.writelines(lines)


def mkd_file():
    """
    FTP创建目录
    :target_directory: 目标目录
    :return:
    """
    ftp = ftplib.FTP()
    ftp.connect(host=FTPConfig.host, port=int(FTPConfig.port))
    ftp.login(user=FTPConfig.user, passwd=FTPConfig.password)
    target_directory = ftp.mkd('/jobs/importer/' + str(int(round(time.time() * 1000))))
    ftp.close()
    return target_directory


def put_file(file_list, target_directory, max_retries=3):
    """
    FTP上传文件，并在失败时重试
    :param file_list: 待上传文件列表
    :param target_directory: 目标目录
    :param max_retries: 最大重试次数
    :return:
    """
    ftp_start_time = time.time()
    retries = 0
    while retries <= max_retries:
        try:
            ftp = ftplib.FTP()
            ftp.connect(host=FTPConfig.host, port=int(FTPConfig.port))
            ftp.login(user=FTPConfig.user, passwd=FTPConfig.password)
            connected = True
        except Exception as e:
            logger.error(f'连接FTP服务器失败: {e}')
            retries += 1
            connected = False
            if retries <= max_retries:
                my_logger.info(f'重试连接FTP服务器, 尝试次数: {retries}')
                time.sleep(5)
            else:
                logger.error('达到最大重试次数，连接失败。')
                exit(-1)

        if connected:
            # 上传文件
            for file in file_list:
                upload_retries = 0
                while upload_retries <= max_retries:
                    try:
                        file_splits = file.split('/')
                        simple_name = file_splits[len(file_splits) - 1]
                        with open(file, 'rb') as fp:
                            cmd = 'STOR %s/%s' % (target_directory, simple_name)
                            ftp.storbinary(cmd, fp)
                        break
                    except Exception as e:
                        logger.error(f'上传文件{file}至FTP失败: {e}')
                        upload_retries += 1
                        if upload_retries > max_retries:
                            logger.error(f'文件{file}达到最大重试次数，上传失败。')
                            exit(-1)
                        else:
                            my_logger.info(f'重试上传文件{file}, 尝试次数: {upload_retries}')
                            time.sleep(5)
            break
    ftp.quit()
    ftp_end_time = time.time()
    ftp_cost_time = ftp_end_time - ftp_start_time
    my_logger.info("文件上传至FTP耗时:%.3f秒" % ftp_cost_time)


def delete_file(file_list, target_directory):
    ftp = ftplib.FTP()
    ftp.connect(host=FTPConfig.host, port=int(FTPConfig.port))
    ftp.login(user=FTPConfig.user, passwd=FTPConfig.password)
    for file in file_list:
        file_splits = file.split('/')
        simple_name = file_splits[len(file_splits) - 1]
        ftp.delete('%s/%s' % (target_directory, simple_name))
    ftp.close()


def extract_and_validate_data(json_data):
    error_message = ""
    # 检查 userKey 字段，如果存在且值为 $notuser，则不需要 userId
    if json_data.get('userKey') != '$notuser' and 'userId' not in json_data:
        error_message += f"缺少userId需指定\n若传主体事件,则数据需字段userKey, 且值为‘$notuser’\n"
        return None, error_message
    # 确保 event,timestamp 字段存在
    elif 'event' not in json_data or 'timestamp' not in json_data:
        error_message += "event或timestamp字段不存在\n"
        return None, error_message

        # 提取字段并创建 DataEvent 对象
    data_event = DataEvent(
        userId=json_data.get('userId', ''),
        event=json_data['event'],
        timestamp=json_data['timestamp'],
        attrs=json_data.get('attrs', {}),
        userKey=json_data.get('userKey', ''),
        eventId=json_data.get('eventId', None),
        dataSourceId=json_data.get('dataSourceId', None)
    )
    return data_event, error_message


def validate_data_event(data_event, eventStart, eventEnd, cstm_keys, cstm_attr_keys):
    error_message = ""
    normal = True

    event = data_event.event
    var_keys = cstm_keys.get(event)
    attr_all = BaseConfig.attr_all

    if event in ['$exit', '$bounce']:
        normal = False
        error_message += f"事件[{event}]为t+1离线计算生成，不支持导入\n"

    if var_keys is None:
        normal = False
        error_message += f"事件[{event}]在GIO平台未定义，请先在系统中定义\n"

    if not str(var_keys).startswith("$"):
        if hasattr(data_event, 'attrs'):
            attrs_customize_error = []
            attrs_bind_error = []
            for attr in data_event.attrs:
                if attr not in attr_all:
                    # 事件属性
                    if attr not in cstm_attr_keys:
                        attrs_customize_error.append(attr)
                    # 事件绑定属性
                    elif var_keys is not None and attr not in var_keys and attr is not None:
                        attrs_bind_error.append(attr)
            if len(attrs_customize_error) > 0 or len(attrs_bind_error) > 0:
                normal = False
                if len(attrs_customize_error) > 0:
                    error_message += f"事件属性[{attrs_customize_error}]在GIO平台未定义，请先在系统中定义\n"
                if len(attrs_bind_error) > 0:
                    error_message += f"不存在事件属性[{attrs_bind_error}]与事件[{event}]的绑定关系\n"

    elif str(var_keys).startswith("$") and var_keys is not None:
        if str(var_keys) not in ["$page", "$visit"]:
            normal = False
            error_message += f"预置事件只支持$page，$visit\n"
        else:
            if hasattr(data_event, 'attrs'):
                attrs_customize_error = []
                attrs_bind_error = []
                for attr in data_event.attrs:
                    if attr not in attr_all:
                        # 事件属性
                        if attr not in cstm_attr_keys:
                            attrs_customize_error.append(attr)
                    # 事件绑定属性
                    elif attr not in var_keys and attr is not None:
                        attrs_bind_error.append(attr)
                if len(attrs_customize_error) > 0 or len(attrs_bind_error) > 0:
                    normal = False
                    if len(attrs_customize_error) > 0:
                        error_message += f"事件属性[{attrs_customize_error}]在GIO平台未定义，请先在系统中定义\n"
                    if len(attrs_bind_error) > 0:
                        error_message += f"不存在事件属性[{attrs_bind_error}]与事件[{event}]的绑定关系\n"

    if not hasattr(data_event, 'timestamp'):
        normal = False
        error_message += "缺少timestamp需指定\n"
    else:
        timestamp = 0
        try:
            timestamp = time_format(str(data_event.timestamp), BaseConfig.timezone)
        except Exception:
            normal = False
            error_message += "timestamp格式错误，请参考数据导入帮助文档\n"

        if (timestamp < eventStart or timestamp > eventEnd) and timestamp != 0:
            normal = False
            error_message += "timestamp时间范围不合法\n"

    return normal, error_message


def count_lines_in_file(paths):
    total_lines = 0
    for path in paths:
        with open(path, 'r') as file:
            lines = sum(1 for line in file)
        total_lines += lines

    return total_lines


def portal_token(new_token):
    """更新 token"""
    ApiConfig.update_token(new_token)



#!/bin/env python3
# -*- coding: UTF-8 -*-

import os
import pathlib
import tempfile
from configparser import ConfigParser

config = ConfigParser()
path = ''
conf_path = str(pathlib.Path(os.path.abspath(__file__)).parent.parent) + "/conf.cfg" if len(str(path)) == 0 else path
config.read(os.path.abspath(conf_path), encoding="UTF-8")


def get_temp_dir_from_config():
    """
    从配置中获取临时存储目录，如果没有配置则使用默认目录
    """
    temp_dir = config['App']['temp_dir']
    return temp_dir


class ApiConfig:
    """A ApiConfig Class"""
    oauth2_uri = config['API']['oauth2_uri']
    token = config['API']['token']
    project_id = config['API']['project_id']

    @classmethod
    def update_token(cls, new_token):
        """Update token"""
        cls.token = new_token


class SSLConfig:
    """SSL证书校验"""
    verify = config['SSL']['verify']


class BaseConfig:
    """A BaseConfig Class"""
    timezone = 'Asia/Shanghai'
    # 所有事件都有的事件属性
    attr_all = ["$session", "$path", "$title", "$referrer_path", "$textValue", "$index", "$xpath",
                "$hyperlink",
                "$duration", "$page_count", "$package", "$platform", "$referrer_domain",
                "$utm_source",
                "$utm_medium", "$utm_campaign", "$utm_term", "$utm_content", "$ads_id",
                "$key_word",
                "$country_code", "$country_name", "$region", "$city", "$browser",
                "$browser_version", "$os",
                "$os_version", "$client_version", "$channel", "$device_brand", "$device_model",
                "$device_type",
                "$device_orientation", "$resolution", "$language", "$referrer_type",
                "$account_id",
                "$domain",
                "$ip", "$user_agent", "$sdk_version", "$location_latitude",
                "$location_longitude",
                "$bot_id", "eventId", "$query",
                "$traffic_source", "$traffic_source_session", "$utm_source_session", "$utm_medium_session",
                "$utm_campaign_session", "$utm_term_session", "$utm_content_session", "$share_title",
                "$target", "$from", "$share_path", "$share_query", "$defer", "$timezone_offset"
                ]


class FTPConfig:
    namespace = config['FTP']['ftp_namespace']
    host = config['FTP']['ftp_host']
    user = config['FTP']['ftp_user']
    password = config['FTP']['ftp_password']
    port = config['FTP']['ftp_port']


class SaasConfig:
    uri = config['SAAS']['saas_uri']
    token = config['SAAS']['saas_token']
    uid = config['SAAS']['saas_uid']


import logging

from importers.meta.data_center import *
import re


def check_key(key):
    """
     校验标识符KEY，不能为空，长度小于100，只可以由字母数字下划线组成，且只能字母开头
    :param key: 字符串
    :return:
    """
    if key == '':
        logging.error("事件名不能为空")
        exit(-1)
    elif len(key) > 100:
        logging.error("事件名<{}>长度超限".format(key))
        exit(-1)
    elif key[0].isdigit() or is_chinese(key) or match(key):
        logging.error("事件名<{}>不合法，只可以由字母数字下划线组成，且只能字母开头~".format(key))
        exit(-1)


def check_name(name, key):
    """
     校验name，长度小于100，当name为空，则默认与标识符一致
    :param name: 名称
    :param key: 标识符
    :return:
    """
    if name != "" and len(name) > 100:
        logging.error("显示名<{}>长度超限~".format(name))
        exit(-1)
    elif len(str(name).strip()) == 0:
        return key
    # elif isexsit_eventvar(name) or isexsit_usertvar(name):
    #     logging.error("名称<{}>已重复~".format(name))
    #     exit(-1)
    else:
        return name


def check_desc(desc):
    """
     校验desc，长度小于150，当name为空，则默认与标识符一致
    :param desc: 描述
    :return:
    """
    if desc != "" and len(desc) > 150:
        logging.error("描述<{}>长度超限~".format(desc))
        exit(-1)
    else:
        return desc


def isexsit_eventvar(name):
    """
     校验事件属性名称是否存在
     :param name: 名称
     :return: True/False
     """
    event_variables = getdataCenterEventVariables()
    for var in event_variables:
        if var['name'] == name:
            return True
    return False


def isexsit_usertvar(name):
    """
     校验用户属性名称是否存在
     :param name: 名称
     :return: True/False
     """
    user_variables = getdataCenterUserVariables()
    for var in user_variables:
        if var['name'] == name:
            return True
    return False


def check_event_valuetype(tp):
    """
     校验事件数据类型
     :param tp: 数据类型
     :return:
     """
    if tp == '':
        logging.error("数据类型<{}>未设置".format(tp))
        exit(-1)
    elif tp not in ('string', 'int', 'double'):
        logging.error("数据类型<{}>不在可选值范围".format(tp))
        exit(-1)


def check_user_valuetype(tp):
    """
     校验用户数据类型
     :param tp: 数据类型
     :return:
     """
    if tp == '':
        logging.error("数据类型<{}>未设置".format(tp))
        exit(-1)
    elif tp not in ('string', 'int', 'date', 'double'):
        logging.error("数据类型<{}>不在可选值范围".format(tp))
        exit(-1)


def check_item_key(key):
    """
     校验主体标识符KEY，不能为空，长度小于100，只可以由字母数字下划线组成，且只能字母开头
    :param key: 字符串
    :return:
    """
    if key == '':
        logging.error("主体标识符不能为空")
        exit(-1)
    elif len(key) > 100:
        logging.error("主体标识符<{}>长度超限".format(key))
        exit(-1)
    elif key[0].isdigit() or is_chinese(key) or match(key):
        logging.error("主体标识符<{}>不合法，只可以由字母数字下划线组成，且只能字母开头~".format(key))
        exit(-1)


def check_item_key_variables(key):
    """
     校验主体标识符KEY，不能为空，长度小于100，只可以由字母数字下划线组成，且只能字母开头
    :param key: 字符串
    :return:
    """
    if key == '':
        logging.error("主体字段不能为空")
        exit(-1)
    elif len(key) > 100:
        logging.error("主体字段<{}>长度超限".format(key))
        exit(-1)
    elif key[0].isdigit() or is_chinese(key) or match(key):
        logging.error("主体字段<{}>不合法，只可以由字母数字下划线组成，且只能字母开头~".format(key))
        exit(-1)


def check_item_valuetype(tp):
    """
     校验主体数据类型
     :param tp: 数据类型
     :return:
     """
    if tp == '':
        logging.error("数据类型<{}>未设置".format(tp))
        exit(-1)
    elif tp not in ('string', 'list', 'int', 'date', 'double'):
        logging.error("数据类型<{}>不在可选值范围".format(tp))
        exit(-1)


def check_event_exsit(key):
    """
     校验事件标识符KEY是否存在
     :param key: 标识符
     :return:
     """
    custom_events = getdataCenterCustomEvents()
    for var in custom_events:
        if var['key'] == key:
            return True, var
    else:
        return False, ''


def check_event_attr_exsit(key):
    """
     校验事件标识符KEY是否存在
     :param key: 标识符
     :return:
     """
    custom_events = getdataCenterCustomEventsAndAttr()
    id_list = []
    for var in custom_events:
        if var['key'] == key:
            if len(var['attributes']) != 0:
                for a in var['attributes']:
                    id_list.append(a['id'])
                return True, var, id_list
            else:
                return True, var, id_list
    else:
        return False, '', []


def check_event_var_exsit(key):
    """
     校验事件属性标识符KEY是否存在
     :param key: 标识符
     :return:
     """
    event_variables = getdataCenterEventVariables()
    for var in event_variables:
        if var['key'] == key:
            return True, var
    else:
        return False, ''


def check_user_var_exsit(key):
    """
     校验用户属性标识符KEY是否存在
     :param key: 标识符
     :return:
     """
    user_variables = getdataCenterUserVariables()
    for var in user_variables:
        if var['key'] == key:
            return True, var
    else:
        return False, ''


def check_item_var_exsit(key):
    """
     校验主体标识符KEY是否存在
     :param key: 标识符
     :return:
     """
    item_variables = getdataCenterItemModels()
    for item in item_variables:
        for attr in item['attributes']:
            if attr['isPrimaryKey'] and attr['key'] == key:
                return True, item['id'], item['name'], item['description']

    return False, '', '', ''


def check_item_key_exsit(key):
    """
     校验新主体字段标识符KEY是否存在
     :param key: 标识符
     :return:
     """
    item_variables = getdataCenterItemModels()
    for item in item_variables:
        for attr in item['attributes']:
            if attr['key'] == key:
                logging.error("字段标识符<{}>全局有重复".format(key))
                exit(-1)


def check_item_name_exsit(name):
    """
     校验新主体名称name是否存在
     :param key: 标识符
     :return:
     """
    item_variables = getdataCenterItemModels()
    for item in item_variables:
        for attr in item['attributes']:
            if attr['name'] == name and name != "":
                logging.error("名称<{}>全局有重复".format(name))
                exit(-1)


def check_item_var_key_exsit(id, key):
    """
     校验主体字段KEY是否存在
     :param key: 标识符
     :return:
     """
    item_variables = getdataCenterItemModels()
    for item in item_variables:
        for attr in item['attributes']:
            if item['id'] == id and attr['isPrimaryKey'] is False and attr['key'] == key:
                return True, attr['name'], attr['valueType'], attr['description'], attr['id']

    return False, '', '', '', ''


def check_item_var_key_data_exsit(old_id):
    """
     校验主体字段KEY是否存在
     :param key: 标识符
     :return:
     """
    item_variables = getdataCenterItemModels()
    key_list = []
    for item in item_variables:
        for attr in item['attributes']:
            if item['id'] == old_id and attr['isPrimaryKey'] is False:
                key_list.append(attr['key'])
    return key_list



def check_bind_event_exsit(key):
    """
     校验事件与事件属性
     :param key: 标识符
     :return:
     """
    custom_events = getdataCenterCustomEvents()
    for var in custom_events:
        if var['key'] == key:
            return True, var['id'], var['description']
    else:
        return False, '', ''


def check_key_name(var_list, key, name):
    """
     校验标识符、名称
     :param var_list: 变量列表
     :param key: 标识符
     :param name: 名称
     :return:
     """
    for var in var_list:
        if var['key'] == key:
            logging.error("标识符<{}>已存在~".format(key))
            exit(-1)
        elif var['name'] == name:
            logging.error("名称<{}>已存在~".format(name))
            exit(-1)


def is_chinese(string):
    """
     检测字符串是否存在汉字
     :param string: 字符串
     :return:True/False
     """
    for ch in string:
        if u'\u4e00' <= ch <= u'\u9fff':
            return True
    return False


def match(str):
    """
     只可以由字母数字下划线组成，且只能字母开头
     :param str: 字符串
     :return:True/False
     """
    mh = re.match(r'[0-9a-zA-Z_]*', str)
    if len(mh.group()) != len(str):
        return True
    else:
        return False


def check_attr(attr):
    """
     校验事件与事件属性
     :param attr: 事件属性
     :return:id_list,key_list
     """
    attrs = str(attr).split(",")
    key_list = []
    id_list = []
    error_list = []
    event_variables = getdataCenterEventVariables()
    for a in attrs:
        for event in event_variables:
            if event['key'] == a:
                id_list.append(event['id'])
                key_list.append(event['key'])
                break
        else:
            error_list.append(a)
    if len(error_list) == len(attrs):
        logging.error("绑定事件属性失败，事件属性:{}不存在,请先创建事件属性~".format(error_list))
        exit(-1)
    elif len(error_list) != 0:
        logging.error("事件属性:{}不存在,请先创建事件属性~".format(error_list))
    return id_list, key_list



#!/bin/env python3
# -*- coding: UTF-8 -*-

# Copyright (c) 2020 growingio.com, Inc.  All Rights Reserved

import json
import os
from json.decoder import JSONDecodeError

import pandas as pd

from importers.common import http_util
from importers.common.common_util import time_str_to_timestamp_of_tz
from importers.common.config_util import BaseConfig
from importers.common.log_util import logger, my_logger
from importers.data_import.data_format_util import *
from importers.data_import.data_model import AdsData, AdsEventsCv, AdsEventsJson
from importers.meta.data_center import getImportJobStatus, trigger_job

ONE_DAY_MILLISECOND = 86400 * 1000  # 1天(24小时)毫秒值


def ads_import(args):
    """
    广告数据导入，按数据格式处理
    """
    # Step one: 校验事件数据基础参数，并预处理
    # 1. 校验时间
    event_start = args.get("event_start")
    if event_start is None:
        logger.error("[-s/--event_start]参数值未指定")
        exit(-1)
    event_end = args.get("event_end")
    if event_end is None:
        logger.error("[-e/--event_end]参数值未指定")
        exit(-1)
    try:
        event_start = (
            time_str_to_timestamp_of_tz(event_start, "%Y-%m-%d", BaseConfig.timezone)
            * 1000
        )
        event_end = (
            time_str_to_timestamp_of_tz(event_end, "%Y-%m-%d", BaseConfig.timezone)
            * 1000
            + ONE_DAY_MILLISECOND
        )
    except (TypeError, ValueError):
        logger.error("[-s/--event_start]或[-e/--event_end]时间参数格式不对,格式为:YYYY-MM-DD")
        exit(-1)
    # 2. 数据源是否为事件
    ds = args.get("datasource_id")
    if "HISTORY_ADS_EVENT" in ds[1]:
        args["datasource_id"] = ds[1]["HISTORY_ADS_EVENT"]
    else:
        logger.error("数据源不属于广告行为类型")
        exit(-1)
    # Step one: 按数据格式处理
    f = str(args.get("format"))
    if "JSON".__eq__(f):
        ads_import_send(
            AdsEventsJson(
                name=args.get("m"),
                path=args.get("path"),
                debug=args.get("debug"),
                format=f,
                datasourceId=args.get("datasource_id"),
                eventStart=event_start,
                eventEnd=event_end,
                jobName=args.get("jobName"),
                clear=args.get("clear"),
            )
        )
    elif "CSV".__eq__(f):
        separator = args.get("separator")
        separator = "," if separator == "" else separator
        ads_import_send(
            AdsEventsCv(
                name=args.get("m"),
                path=args.get("path"),
                debug=args.get("debug"),
                format=f,
                datasourceId=args.get("datasource_id"),
                attributes=args.get("attributes"),
                separator=separator,
                skipHeader=args.get("skip_header"),
                eventStart=event_start,
                eventEnd=event_end,
                jobName=args.get("jobName"),
                clear=args.get("clear"),
            )
        )
    elif "TSV".__eq__(f):
        separator = args.get("separator")
        separator = "\t" if separator == "" else separator
        ads_import_send(
            AdsEventsCv(
                name=args.get("m"),
                path=args.get("path"),
                debug=args.get("debug"),
                format=f,
                datasourceId=args.get("datasource_id"),
                attributes=args.get("attributes"),
                separator=separator,
                skipHeader=args.get("skip_header"),
                eventStart=event_start,
                eventEnd=event_end,
                jobName=args.get("jobName"),
                clear=args.get("clear"),
            )
        )


# SV格式(CSV、TSV)
def ads_sv_import_prepare_process(
    attributes, paths, skip_header, separator, qualifier, eventStart, eventEnd, debug
):
    """
    1. 校验数据基本信息
    2. SV格式数据转换为数据对象
    """
    # Step 1: 校验有无attributes,有无重复列名
    if attributes is None:
        logger.error("[-attr/--attributes]参数值不存在")
        exit(-1)

    commonlist = ["key", "timestamp", "utm_source", "utm_campaign"]
    ads_attr_keys = [
        "ip",
        "os",
        "osv",
        "platform",
        "ads_id",
        "$ad_group",
        "$ad_plan",
        "$ad_creative",
        "$utm_term",
        "$utm_medium",
        "$utm_content"
    ]
    cols = str(attributes).split(",")
    res = [i for i in cols if i not in commonlist]

    attr_all = BaseConfig.attr_all
    total_error = 0
    for path in paths:
        count = 0
        error = 0
        if skip_header:
            replace_first_line_with_header(path, cols, separator)
        # 读取文件第一行，与预期表头比较
        with open(path, "r", encoding="utf8") as f:
            first_line = f.readline().strip()
            expected_header = separator.join(cols)
            if first_line != expected_header:
                add_header_to_file(path, cols, separator)

        skh = True
        with open(path, "r", encoding="utf8") as f:
            for line in f:
                count += 1
                line = line.replace("\n", "").replace("\\t", "\t")
                # Step 2: 校验数据header列是否一致，数量和顺序
                if skh is True and count == 1:
                    header_normal = True
                    error_key = []
                    for key in res:
                        if key not in ads_attr_keys and key not in attr_all:
                            error_key.append(key)
                    if len(error_key) > 0:
                        logger.error(
                            f"[-attr/--attributes]参数值列:事件属性{error_key}在GIO平台未定义"
                        )
                        header_normal = False
                    if not header_normal:
                        logger.error("header校验失败！")
                        return False
                else:
                    # Step 2: 数据列是否一致
                    line_normal = True
                    if check_sv_header_col_count(cols, line.split(separator)) is False:
                        remove_first_line_from_csv(path)
                        logger.error(
                            f"第{count}行:导入文件[{path}]的列数和参数值列数不一致\n请检查文件分隔符，并通过参数[-sep]指定"
                        )
                        return False
                    if not line_normal:
                        continue
        if debug:
            df = pd.read_csv(
                path,
                sep=separator,
                header=0 if skh else None,
                names=cols if skh else None,
            )
            for index, row in df.iterrows():
                lineNumber = index + 1
                count += 1
                col_value = row.to_dict()
                # 提取必要的字段
                ads_event, error_message = ads_extract_and_validate_data(col_value)
                if ads_event == None:
                    logger.error(
                        f"第{lineNumber}行: 文件[{path}]数据[{row.to_string(index=False)}]:\n{error_message}"
                    )
                    error += 1
                    continue

                # 执行数据对象的校验
                is_valid, error_message = valid_ads_event(
                    ads_event, eventStart, eventEnd
                )
                if not is_valid:
                    logger.error(
                        f"第{lineNumber}行: 文件[{path}]数据[{row.to_string(index=False)}]:\n{error_message}"
                    )
                    error += 1
                    if not skip_header:
                        logger.error("请检查文件表头数据是否新增以参数[attr]参数为准的表头,请根据信息修改文件内容或参数信息")
                    continue
                if count % 500000 == 0:
                    my_logger.info(f"已经校验{count}条数据......")

            if error > 0:
                logger.warn(f"文件[{path}]导入失败，共发现[{error}]个校验错误")
                total_error += error

        if total_error > 0:
            logger.warn(f"总共发现[{total_error}]个校验错误")
            return False
        else:
            return True


def ads_import_send(events):
    """
    ADS行为导入，Json格式数据处理
    """
    # Step 1: 执行Debug
    f = events.format
    if events.debug:
        if "JSON".__eq__(f):
            checkResult = ads_events_debug_process(
                events.path, events.eventStart, events.eventEnd
            )
            if checkResult is not True:
                logger.error("Debug校验未通过")
                exit(-1)
        else:
            checkResult = ads_sv_import_prepare_process(
                attributes=events.attributes,
                paths=events.path,
                skip_header=events.skipHeader,
                separator=events.separator,
                qualifier=events.qualifier,
                eventStart=events.eventStart,
                eventEnd=events.eventEnd,
                debug=events.debug,
            )
            if checkResult is not True:
                logger.error("Debug校验未通过")
                exit(-1)

    if len(str(events.jobName)) == 0 or events.jobName is None:
        jobName = f"Python_ads_events_{time.strftime('%Y%m%d_%H%M%S', time.localtime(time.time()))}"
    else:
        jobName = events.jobName
    # Step 2: 创建导入任务
    if "JSON".__eq__(f):
        job_info = create_task_json(
            events.eventStart, events.eventEnd, events.datasourceId, jobName
        )
    else:
        job_info = create_task_sv(
            events.eventStart,
            events.eventEnd,
            events.datasourceId,
            jobName,
            events.separator,
        )

    # 任务名重复时，获取不到job信息时，程序直接结束
    if job_info is None:
        logger.error("job_info为空，无法创建导入任务")
        exit(-1)
    else:
        my_logger.info(f"创建导入任务: {job_info}")
    direct = job_info["argument"]["directory"]
    # Step 3: 上传数据到FTP
    my_logger.info(f"文件开始上传至FTP")
    put_file(events.path, direct)

    # Step 4: 启动导入任务
    start_time = time.time()
    trigger_job(job_info["id"])
    my_logger.info(f"开始执行导入任务")
    flag = True
    while flag:
        eventImportJob = getImportJobStatus(job_info["id"])
        if eventImportJob is not None:
            stage = eventImportJob["stage"]
            error = eventImportJob["error"]
            if stage is not None and stage.__eq__("FINISH"):
                end_time = time.time()
                cost_time = end_time - start_time
                my_logger.info("导入成功")
                delete_file(events.path, direct)
                my_logger.info("执行导入任务耗时:%.3f秒" % cost_time)
                flag = False
            elif stage is not None and stage.__eq__("ERROR"):
                if error is not None:
                    message = error.get("message", "No message available")
                else:
                    message = "Error object is None"
                end_time = time.time()
                cost_time = end_time - start_time
                logger.error(f"导入失败,错误信息为[ {message} ] \n FTP文件路径: {direct}")
                my_logger.info("执行导入任务耗时:%.3f秒" % cost_time)
                exit(-1)
        if flag:
            my_logger.info(f"等待任务完成......")
            time.sleep(10)


def create_task_json(start, end, tid, name):
    """
    创建josn任务
    """
    if len(str(name)) == 0:
        body = """{ "operationName":"createEventImportJob", "variables":{ "fileType":"json", "tunnelId":"%s", 
            "timeRange":"abs:%s,%s", "createType":"PYTHON" }, "query":"mutation createEventImportJob($tunnelId: HashId!, 
            $timeRange: String, $fileType: String, $createType: String) { createEventImportJob(tunnelId: $tunnelId, 
            timeRange: $timeRange, fileType: $fileType, createType: $createType) { id name argument { directory 
            __typename } __typename } }" }""" % (
            tid,
            start,
            end,
        )
    else:
        body = """{ "operationName":"createEventImportJob", "variables":{ "fileType":"json", "tunnelId":"%s", 
            "timeRange":"abs:%s,%s", "name":"%s", "createType":"PYTHON" }, "query":"mutation createEventImportJob($name: 
            String, $tunnelId: HashId!, $timeRange: String, $fileType: String, $createType: String) { 
            createEventImportJob(name: $name, tunnelId: $tunnelId, timeRange: $timeRange, fileType: $fileType, 
            createType: $createType) { id name argument { directory __typename } __typename } }" }""" % (
            tid,
            start,
            end,
            name,
        )

    resp = http_util.send_graphql_post(body.encode("utf-8"))
    try:
        return resp["createEventImportJob"]
    except TypeError:
        logger.error("自定义任务名称已存在！")


def create_task_sv(start, end, tid, name, separator):
    """
    创建csv/tsv任务
    """
    separator = separator.replace("\t", "\\\\t")
    if len(str(name)) == 0:
        body = """{ "operationName":"createEventImportJobV2", "variables":{ "fileType":"csv", "tunnelId":"%s", 
            "timeRange":"abs:%s,%s", "createType":"PYTHON", "csvSeparator":"%s" }, "query":"mutation 
            createEventImportJobV2($tunnelId: HashId!, $timeRange: String, $fileType: String, $createType: String, 
            $csvSeparator: String) { createEventImportJobV2(tunnelId: $tunnelId, timeRange: $timeRange, 
            fileType: $fileType, createType: $createType, csvSeparator: $csvSeparator) { id name argument { directory 
            __typename } __typename } }" }""" % (
            tid,
            start,
            end,
            separator,
        )
    else:
        body = """{ "operationName":"createEventImportJobV2", "variables":{ "fileType":"csv", "tunnelId":"%s", 
            "timeRange":"abs:%s,%s", "name":"%s", "createType":"PYTHON", "csvSeparator":"%s" }, "query":"mutation 
            createEventImportJobV2($name: String, $tunnelId: HashId!, $timeRange: String, $fileType: String, 
            $createType: String, $csvSeparator: String) { createEventImportJobV2(name: $name, tunnelId: $tunnelId, 
            timeRange: $timeRange, fileType: $fileType, createType: $createType, csvSeparator: $csvSeparator) { id 
            name argument { directory __typename } __typename } }" }""" % (
            tid,
            start,
            end,
            name,
            separator,
        )

    resp = http_util.send_graphql_post(body.encode("utf-8"))
    try:
        return resp["createEventImportJobV2"]
    except TypeError:
        logger.error("自定义任务名称已存在！")


def ads_events_debug_process(paths, eventStart, eventEnd):
    """
    获客广告数据行为导入Debug
    1、校验文件的数据内容合法性, 是否缺失必要字段(userId,event,timestamp)
    2、校验文件的数据时间范围合法性
    3、校验自定义事件在GIO平台是否定义
    """
    count = 0
    error_count = 0
    correct_count = 0  # 正确行数
    error_dirs = []
    for path in paths:
        my_logger.info(f"start check file: {path}")

        # 1 重命名 文件名 加后缀 _tmp
        names = path.rsplit("/", 1)
        path_tmp = path + "_tmp"
        os.rename(path, path_tmp)  # 如果新文件名的文件 存在 则覆盖 ok的

        names2 = names[0].rsplit("/", 1)
        grandpa_path = names2[0]
        father_path = names2[-1]
        error_file_name = (
            names[-1] + "_" + str(int(round(time.time() * 1000))) + "_error"
        )

        # 错误空文件名
        dir_error = f"{grandpa_path}/{father_path}_error"
        error_dirs.append(dir_error)
        path_error = f"{dir_error}/{error_file_name}"
        if not os.path.exists(dir_error):
            os.makedirs(dir_error)

        # 2 打开 每个文件， 加后缀 _tmp 文件
        with open(path_tmp, "r", encoding="utf8") as f:
            # 打开正确空文件
            with open(path, "w", encoding="utf8") as f_correct:
                # 打开错误空文件
                with open(path_error, "w", encoding="utf8") as f_error:
                    current_file_error_lines = 0
                    for line in f:
                        count = count + 1
                        source_line = (
                            line  # 把原始行数据 保存到 source_line 变量中，后面判断 正常 异常 ，再写入到相应的文件中
                        )
                        line = line.replace("\n", "")
                        if not line == "":
                            try:
                                json_data = json.loads(line.strip())
                            except JSONDecodeError:
                                logger.error(
                                    f"第{count}行:文件[{path}]数据[{line}]:\n数据非JSON格式\n"
                                )
                                error_count += 1
                                current_file_error_lines += 1
                                f_error.write(source_line)  # 写入异常文件
                                continue

                            ads_event, message = ads_extract_and_validate_data(
                                json_data
                            )
                            if ads_event is None:
                                logger.error(
                                    f"第{count}行:文件[{path}]数据[{line}]\n" f"{message}"
                                )
                                error_count += 1
                                current_file_error_lines += 1
                                f_error.write(source_line)  # 写入异常文件
                                continue
                            is_valid, error_message = valid_ads_event(
                                ads_event, eventStart, eventEnd
                            )
                            if not is_valid:  # 异常
                                logger.error(
                                    f"第{count}行:文件[{path}]数据[{line}]:\n"
                                    f"{error_message}"
                                )
                                error_count += 1
                                current_file_error_lines += 1
                                f_error.write(source_line)  # 写入异常文件
                            else:  # 正常
                                f_correct.write(source_line)  # 写入正常文件
                                correct_count = correct_count + 1
                            if count % 500000 == 0:
                                my_logger.info(f"已经校验{count}条数据......")
                        else:
                            logger.warn(f"第{count}行为空，跳过该行")
                f_error.close()  # 关闭 异常文件
            f_correct.close()  # 关闭 正常文件
        f.close()  # 关闭 带有后缀 _tmp的文件

        # 删除 带有后缀 _tmp的文件
        os.remove(path_tmp)
        # 判断 若 异常文件空白 行数=0，则 删除 异常文件
        if current_file_error_lines == 0:
            os.remove(path_error)

    # 判断 若 异常 文件夹 空白，则 删除 该文件夹
    for dir in error_dirs:
        if len(os.listdir(dir)) == 0:
            os.removedirs(dir)

    if error_count == 0:
        my_logger.info(f"本次共校验[{count}]行数据")
    else:
        my_logger.info(
            f"本次共校验[{count}]行数据,其中校验失败[{error_count}]行数据,异常数据已剪切到如下目录中 {error_dirs}"
        )

    if correct_count == 0:
        my_logger.info(f"由于本次正确数据0条，故不生成导数任务。")
        return False
    else:
        return True


def valid_ads_event(adsEvent, eventStart, eventEnd):
    error_message = ""
    try:
        timestamp = time_format(str(adsEvent.timestamp), BaseConfig.timezone)
    except Exception:
        error_message += "timestamp格式错误，请参考数据导入帮助文档\n"
        return False, error_message
    if (timestamp < eventStart or timestamp > eventEnd) and timestamp != 0:
        error_message += "timestamp时间范围不合法\n"
        return False, error_message
    return True, None
    


def ads_extract_and_validate_data(json_data):
    error_message = ""
    columns = ["key", "timestamp", "utm_campaign", "utm_source"]
    for column in columns:
        if column not in json_data:
            error_message += column + "字段不存在\n"

    if error_message != "":
        return None, error_message

    keyValue = json_data.get("key")
    if keyValue not in ["$ads_click", "$ads_imp"]:
        error_message += f"key 必须是 $ads_click 或者 $ads_imp\n"
        return None, error_message

    # 提取字段并创建 AdsEvent 对象
    ads_event = AdsData(
        key=json_data["key"],
        utmSource=json_data["utm_source"],
        utmCampaign=json_data["utm_campaign"],
        timestamp=json_data["timestamp"],
        attrs=json_data.get("attrs", {}),
        dataSourceId=json_data.get("dataSourceId", None),
    )
    return ads_event, error_message



import json

from importers.common.http_util import send_graphql_post


# 获取custom_events表(id,key,name,description)数据
def getdataCenterCustomEvents():
    """
        获取 CustomEvents (id,key,name,description)
    """
    body = """{"query":"query MyQuery {  dataCenterCustomEvents {    id   key   name   description}}","variables":null,"operationName":"MyQuery"}"""
    content = send_graphql_post(body)
    data_info = content['dataCenterCustomEvents']
    return data_info


def getdataCenterCustomEventsAndAttr():
    """
        获取 CustomEvents (id,key,name,description,attributes(id,key))
    """
    body = """{"query":"query MyQuery {  dataCenterCustomEvents {    id   key   name   description   attributes {id  key}}}","variables":null,"operationName":"MyQuery"}"""
    content = send_graphql_post(body)
    data_info = content['dataCenterCustomEvents']
    return data_info


def getdataCenterUserVariables():
    """
        获取 UserVariables (id,key,name,description,valueType)
    """
    body = """{"query":"query MyQuery {  dataCenterUserVariables {    id   key   name   description   valueType}}","variables":null,"operationName":"MyQuery"}"""
    content = send_graphql_post(body)
    data_info = content['dataCenterUserVariables']
    return data_info


def getdataCenterEventVariables():
    """
        获取 EventVariables (id,key,name,description,valueType)
    """
    body = """{"query":"query MyQuery {dataCenterEventVariables {    id   key   name   description   valueType}}","variables":null,"operationName":"MyQuery"}"""
    content = send_graphql_post(body)
    data_info = content['dataCenterEventVariables']
    return data_info


# 获取元数据
def getMetaData():
    """
        获取元数据
    """
    body = """{"query":"query MyQuery {dataCenterCustomEvents {    key    name    description  }  dataCenterEventVariables {    key    name    
    valueType    description  }  dataCenterUserVariables {    key    name    valueType    description  }}","variables":null,"operationName":"MyQuery"}"""
    content = send_graphql_post(body)
    data_info = content
    return data_info


def getBindEvent():
    """
        获取绑定事件和关联事件属性
    """
    body = """{"query":"query MyQuery {  dataCenterCustomEvents {    key    attributes {      key    }  }}","variables":null,"operationName":"MyQuery"}"""
    content = send_graphql_post(body)
    data_info = content
    return data_info


def getTunnels():
    """
        获取 tunnels(id  key   type   config)
    """
    body = """{"query":"query MyQuery { tunnels {id  key   type   config  }}","variables":null,"operationName":"MyQuery"}"""
    content = send_graphql_post(body)
    tunnels = content['tunnels']
    t_res = {}
    for t in tunnels:
        res = {}
        if t['type'] == 'FILE':
            res[eval(t['config'])['type']] = t['id']
            t_res[t['key']] = res
    return t_res


def getClearUsers():
    """
        获取待删除用户列表
    """
    body = """{"query":"query MyQuery {clearUsers {id projectId stage}}","variables":null,"operationName":"MyQuery"}"""
    content = send_graphql_post(body)
    clearusers = content['clearUsers']
    res = {}
    for user in clearusers:
        if (user['stage'] == 'WAITING'):
            res[user['id']] = str(user['projectId'])
    return res


def getGidByUserId(userid):
    """
        获取用户ID对应Gid
    """
    body = """{
        "operationName":"getGidByUser",
        "variables":{
            "userId":"%s"
        },
        "query":"query getGidByUser($userId: String!) {
          getGidByUser(userId: $userId) 
            }
        "}
    """ % (userid)
    content = send_graphql_post(body)
    return content['getGidByUser']


def getGidAllData(gid):
    """
        获取Gid下所有埋点、用户属性数据
    """
    body = """{
        "operationName":"getAllUserData",
        "variables":{
            "gioId":"%s"
        },
        "query":"query getAllUserData($gioId: Long!) {
            getAllUserData(gioId: $gioId)
        }
    "}
    """ % (gid)
    content = send_graphql_post(body)
    return content['getAllUserData']


def updateClearUserJobStatus(id):
    """
        更新待删除用户状态
    """
    body = """{
        "operationName":"updateClearUser",
        "variables":{
            "id":"%s",
            "clearUser":{
                "stage":"RUNNING"
            }
        },
        "query":"mutation updateClearUser($id: HashId!,$clearUser:ClearUserInput!) {
            updateClearUser(id:$id,clearUser:$clearUser) {
                stage
            }
        }
    "}
    """ % (id)
    content = send_graphql_post(body)
    return content['updateClearUser']['stage']


def deleteClearUserData(gid):
    """
        删除Gid下埋点、用户属性数据
    """
    body = """{
        "operationName":"deleteClearUserData",
        "variables":{
           "gioId":"%s"
        },
        "query":"mutation deleteClearUserData($gioId: Long!) {
            deleteClearUserData(gioId: $gioId)
        }
    "}
    """ % (gid)
    content = send_graphql_post(body)
    return content['deleteClearUserData']


def checkUserExits(userid):
    """
        检查用户是否存在
    """
    body = """{
        "operationName":"checkUserExists",
        "variables":{
            "clearUserId":"%s"
        },
        "query":"query checkUserExists($clearUserId: String!) {
            checkUserExists(clearUserId: $clearUserId)
        }
    "}
    """ % (userid)
    content = send_graphql_post(body)
    return content['checkUserExists']


def createClearUser(userid):
    """
        添加待删除用户
    """
    body = """{
        "operationName":"createClearUser",
        "variables":{
            "clearUser":{
            "userId":"%s"
            }
        },
        "query":"mutation createClearUser($clearUser: ClearUserInput!) {
        createClearUser(clearUser: $clearUser) {
           id   userId   stage    __typename 
         }
        }
    "}
    """ % (userid)
    content = send_graphql_post(body)
    return content['createClearUser']


def trigger_job(id):
    """
      触发任务
    """
    body = '''{
      "operationName":"executeJob",
      "variables":{"id":"%s"},
      "query":"mutation executeJob($id: HashId!) {  
        executeJob(id: $id) {
            id    
            stage    
            __typename  
        }
      }"
    }''' % id
    send_graphql_post(body)


def getImportJobStatus(id):
    body = '''{ 
    "operationName": "eventImportJob", 
    "variables": {"jobId": "%s"}, 
    "query": "query eventImportJob($jobId: HashId!) { 
      eventImportJob(jobId: $jobId) {\n    id\n    name\n    creator\n    creatorId\n    createdAt\n 
       updater\n    updaterId\n    updatedAt\n    description\n    dagRunId\n    parameter {\n      tunnelId\n      
       timeRange\n      fileType\n      __typename\n    }\n    argument {\n      timeRange\n      directory\n      
       __typename\n    }\n    stage\n    error {\n      code\n      message\n      __typename\n    }\n    type\n    
       __typename\n  }\n}\n" 
    }''' % id
    content = send_graphql_post(body)
    eventImportJob = content['eventImportJob']
    return eventImportJob


def getdataCenterItemModels():
    """
        获取 ItemVariables (id,key,name,isPrimaryKey,updatedAt)
    """
    body = """{"operationName":"dataCenterItemModels","variables":{},"query":"query dataCenterItemModels {\n  dataCenterItemModels {\n    id\n    name\n    creator\n    creatorId\n    createdAt\n    updater\n    updaterId\n    updatedAt\n    description\n    eventVariableId\n    attributes {\n      id\n      name\n      key\n      valueType\n      isPrimaryKey\n    description\n      updatedAt\n      __typename\n    }\n    __typename\n  }\n}\n"}"""
    content = send_graphql_post(body)
    data_info = content['dataCenterItemModels']
    return data_info



#!/bin/env python3
# -*- coding: UTF-8 -*-

# Copyright (c) 2020 growingio.com, Inc.  All Rights Reserved
import argparse
import os
import pathlib
import sys

sys.path.append(str(pathlib.Path(os.path.abspath(__file__)).parent.parent))
from importers.meta.meta_create import *
from importers.clear_data.clear_data import globalMetricJob


def parse_args():
    parser = argparse.ArgumentParser(formatter_class=argparse.ArgumentDefaultsHelpFormatter)
    parser.add_argument('-m',
                        help='必填参数. 触发任务名',
                        required=True, metavar="")
    parser.add_argument('-s', '--start_time',
                        help='可选参数. 数据起始时间,导入用户行为数据时指定',
                        default='',
                        type=str)
    parser.add_argument('-e', '--end_time',
                        help='可选参数. 数据结束时间,导入用户行为数据时指定',
                        default='',
                        type=str)
    args = parser.parse_args()
    return args.__dict__


def main():
    args = parse_args()
    m = args.get("m")
    if 'GlobalMetricDailyPipeline'.__eq__(m):
        globalMetricJob(args.get("start_time"), args.get("end_time"))
    else:
        logging.error("请确认填写项目名！")
        exit(-1)


if __name__ == '__main__':
    main()



#!/bin/env python3
# -*- coding: UTF-8 -*-

# Copyright (c) 2020 growingio.com, Inc.  All Rights Reserved


"""
用户属性
"""
import argparse
import os
import pathlib
import sys

project_root = str(pathlib.Path(__file__).resolve().parent.parent)
if project_root not in sys.path:
    sys.path.insert(0, project_root)
from importers.data_import.data_events import events_import
from importers.data_import.data_user_variable import user_variables_import
from importers.data_import.data_ads import ads_import
from importers.common.common_util import get_all_file
from importers.common.log_util import logger, my_logger
from importers.meta.data_center import getTunnels
from importers.data_import.data_item_variable import item_variables_import


def parse_args():
    parser = argparse.ArgumentParser(formatter_class=argparse.ArgumentDefaultsHelpFormatter)
    parser.add_argument('-m',
                        help='必填参数. 用户属性数据导入:user_variables，用户行为数据导入:events，主体数据导入:item_variables，广告数据:ads.',
                        required=True,
                        default='',
                        type=str)
    parser.add_argument('-p', '--path',
                        help='必填参数. 需要导入的数据所在的路径',
                        required=True,
                        default='',
                        type=str)
    parser.add_argument('-ds', '--datasource_id',
                        help='必填参数. 数据源ID.',
                        required=True,
                        default='',
                        type=str)
    parser.add_argument('-f', '--format',
                        help='可选参数. 导入数据格式,目前支持JSON,CSV,TSV三种格式.',
                        default='JSON',
                        type=str)
    parser.add_argument('-item_key',
                        help='必填参数. item_key',
                        required=False,  # 根据-m参数判断是否必填
                        default='',
                        type=str)
    # 暂时取消
    parser.add_argument('-d', '--debug',
                        help='可选参数. True-导入数据全量校验,false-不校验数据是否平台定义立即创建导入任务进入导入队列.',
                        default=True)
    parser.add_argument('-s', '--event_start',
                        help='可选参数. 数据起始时间,导入用户行为数据时指定',
                        default='',
                        type=str)
    parser.add_argument('-e', '--event_end',
                        help='可选参数. 数据结束时间,导入用户行为数据时指定',
                        default='',
                        type=str)
    parser.add_argument('-qf', '--qualifier',
                        help='可选参数. 文本限定符.',
                        default='"',
                        type=str)
    parser.add_argument('-sep', '--separator',
                        help='可选参数. 文本分割符.',
                        default='',
                        type=str)
    parser.add_argument('-skh', '--skip_header',
                        help='可选参数. 设置则自动跳过首行.',
                        action='store_true')
    parser.add_argument('-attr', '--attributes',
                        help='可选参数. 导入文件的各列按顺序映射到属性名，英文逗号分隔.',
                        default='',
                        type=str)
    parser.add_argument('-j', '--jobName',
                        help='可选参数. 可以更改默认jobName',
                        default='',
                        type=str)
    parser.add_argument('-c', '--clear',
                        help='可选参数. True-导入数据成功后清理掉FTP上数据,False-导入数据成功后不清理掉FTP上数据.',
                        default=False)
    parser.add_argument('-v', '--version', action='version', version='Gio_DataImporter_1.0')
    args = parser.parse_args()
    return args.__dict__


def execute_importer(args):
    # 1. 校验导入文件
    p = args.get('path')
    ps = []
    if os.path.exists(p):
        ps.extend(get_all_file(p))
    else:
        logger.error("需要导入的数据文件不存在")
        exit(-1)
    args['path'] = ps
    # 2. 校验Debug参数
    # 暂时取下
    d = str(args.get('debug')).upper()
    if 'TRUE'.__eq__(d):
        d = True
    elif 'FALSE'.__eq__(d):
        my_logger.info("离线导入工具该任务数据跳过校验")
        d = False
    else:
        logger.error("[-d/--debug]参数值不对")
        exit(-1)
    args['debug'] = d
    # 3. 校验数据格式
    f = str(args.get('format')).upper()
    if 'JSON'.__eq__(f) is False and 'CSV'.__eq__(f) is False and 'TSV'.__eq__(f) is False:
        logger.error("目前支持JSON,CSV,TSV三种格式")
        exit(-1)
    args['format'] = f
    # 4. 校验数据源
    m = args.get('m')
    tunnels = getTunnels()
    if args.get('datasource_id') not in tunnels:
        logger.error("数据源不存在")
        exit(-1)
    args['datasource_id'] = [args.get('datasource_id'), tunnels[args.get('datasource_id')]]
    # Step three:  按导入模块处理
    if 'user_variables'.__eq__(m):
        user_variables_import(args)
    elif 'events'.__eq__(m):
        events_import(args)
    elif 'item_variables'.__eq__(m):
        item_variables_import(args)
    elif 'ads'.__eq__(m):
        ads_import(args)
    else:
        logger.warn("目前只支持用户行为数据、用户属性数据导入和主体导入")
        exit(-1)


def do_importer(params):
    my_logger.info("Data Importer Start")
    # 设置默认参数值
    default_params = {
        'm': '',
        'path': '',
        'datasource_id': '',
        'format': 'JSON',  # 默认为 'JSON'
        'item_key': '',  # 默认为空字符串
        'debug': True,  # 默认为 True
        'event_start': '',  # 默认为空字符串
        'event_end': '',  # 默认为空字符串
        'qualifier': '"',  # 默认为双引号
        'separator': '',  # 默认为空字符串
        'skip_header': False,  # 默认为 False
        'attributes': '',  # 默认为空字符串
        'jobName': '',  # 默认为空字符串
        'clear': False,  # 默认为 False
    }

    # 如果 params 中有相同的键，则 params 的值会覆盖 default_params 中的默认值
    combined_params = {**default_params, **params}

    execute_importer(combined_params)
    my_logger.info("Data Importer Finish")


def main():
    # Step one: 解析命令行参数
    args = parse_args()
    logger.debug(f"解析命令行参数: {args}")
    # Step one: 校验基础参数,并预处理参数
    execute_importer(args)


if __name__ == '__main__':
    my_logger.info("Data Importer Start")
    main()
    my_logger.info("Data Importer Finish")



#!/bin/env python3
# -*- coding: UTF-8 -*-
import os
import subprocess

import curlify
import requests

import urllib3
import json
import logging

from importers.common.config_util import ApiConfig, SaasConfig, SSLConfig

http = urllib3.PoolManager()


def send_graphql_post(body):
    """
     graphql post请求
    :param body: 请求graphql body
    :return: 响应数据
    """
    headers = {'Content-Type': 'application/json', 'Authorization': ApiConfig.token,
               'X-Project-Id': ApiConfig.project_id,
               'X-Product-Unique-Id': 'DC', 'User-Agent': 'gio-importer'}
    r = requests.post(ApiConfig.oauth2_uri + "/v3/graphql", headers=headers, data=body, verify=SSLConfig.verify)
    logging.debug('request curl: {}'.format(curlify.to_curl(r.request)))
    if r.status_code == 200:
        content = json.loads(r.content)
        if content["data"] is not None:
            return content['data']
        else:
            logging.error('Graphql请求错误,{}'.format(content['errors']))
            exit(-1)
    else:
        logging.error('request body is {}'.format(body))
        logging.error('response content: {}'.format(r.content.decode('utf-8')))
        re = input("Graphql请求超时或失败,建议检查配置文件的相关token或是否重新请求，请输入yes/no:")
        if str(re).lower() == 'yes':
            send_graphql_post(token, body)
        else:
            return


def send_rest_get(url, params):
    """
     http post请求
    :param url: 请求url
    :param params: 请求参数
    :return: 响应数据
    """
    headers = {'Content-Type': 'application/json', 'Authorization': SaasConfig.token}
    # headers = {'Content-Type': 'application/json'}
    r = requests.get(SaasConfig.uri + url, headers=headers, params=params)
    if r.status_code == 200:
        content = json.loads(r.text)
        return content


def put_file_ftp(file_list, target_directory):
    headers = {'Authorization': 'Bearer ' + ApiConfig.token}
    for file in file_list:
        file_splits = file.split('/')
        simple_name = file_splits[len(file_splits) - 1]
        with open(file, 'rb') as fp:
            file_data = fp.read()
            http.request('POST', ApiConfig.oauth2_uri + '/upload', headers=headers,
                         fields={'file': (simple_name, file_data), 'path': target_directory})


def put_file_hdfs(file_list, target_directory):
    for file in file_list:
        os.system(f"hdfs dfs -put {file} {target_directory}")



#!/bin/env python3
# -*- coding: UTF-8 -*-

import logging.config
import yaml
import os
import pathlib

path = os.path.abspath(str(pathlib.Path(os.path.abspath(__file__)).parent.parent))

from importers.common.common_util import create_dir

default_log_dir = f'{path}/logs'
default_log_file = 'importer.log'
default_log_path = f'{default_log_dir}/{default_log_file}'

with open(path + '/log_conf.yaml', 'rt') as f:
    config = yaml.safe_load(f.read())
    config_log_path = config['handlers']['file']['filename']
    if default_log_file.__eq__(config_log_path):
        config['handlers']['file']['filename'] = f'{default_log_path}'
        create_dir(default_log_dir)
    else:
        config_log_dir, config_log_file = os.path.split(pathlib.Path(config_log_path))
        create_dir(config_log_dir)
    logging.config.dictConfig(config)

logger = logging.getLogger(__name__)

my_logger = logging.getLogger('my_logger')
my_logger.setLevel(logging.INFO)



import json

import requests

from importers.common.common_util import date_range
from importers.common.config_util import ApiConfig
from importers.common.log_util import logger, my_logger
import time

from importers.meta.data_center import updateClearUserJobStatus, getClearUsers, checkUserExits, createClearUser


def clearUserData(execute):
    if str(execute).upper() == 'TRUE':
        users = getClearUsers()
        projectIds = set()
        for id, value in users.items():
            projectIds.add(value)
            assert updateClearUserJobStatus(id) == 'RUNNING'
        for id in projectIds:
            headers = {'Content-Type': 'application/json', 'Authorization': ApiConfig.token}
            ftpPath = '/clearuser/' + str(int(round(time.time() * 1000)))
            body = {'pipelineDefinitionId': 'ClearUserPipeline', 'identity': 'gio',
                    'conf': {'$ftpPath': ftpPath, '$projectId': id}}
            r = requests.post(ApiConfig.oauth2_uri + '/data-server/scheduler/pipeline/running', headers=headers,
                              data=json.dumps(body))
            if r.status_code == 200:
                my_logger.info("提交清理用户删除离线任务")
                my_logger.info("待删除用户数据存放FTP位置为:{}".format('/ftp/admin' + ftpPath))
    elif str(execute).upper() == 'FALSE':
        users = getClearUsers()
        for user in users:
            assert updateClearUserJobStatus(user) == 'RUNNING'
        my_logger.info("等待天任务清理用户删除任务")
        my_logger.info("待删除用户数据将存放FTP位置为:{}".format('/ftp/admin/clearuser'))


def clearUserMeta(userids):
    users = str(userids).split(",")
    for user in users:
        if checkUserExits(user):
            continue
        else:
            logger.error("用户:{} 不存在！".format(user))
            exit(-1)
    for user in users:
        createClearUser(user)
    my_logger.info("添加待删除用户完成，用户为:{}。".format(userids))


def globalMetricJob(start_time, end_time):
    headers = {'Content-Type': 'application/json', 'Authorization': ApiConfig.token}
    time_list = date_range(start_time, end_time)
    if len(time_list) > 30:
        logger.error("超出一个月任务需要重跑计算")
        exit(-1)
    for time in time_list:
        body = {'pipelineDefinitionId': 'GlobalMetricDailyPipeline', 'identity': 'gio', 'conf': {'$startPos': time}}
        r = requests.post(ApiConfig.oauth2_uri + '/data-server/scheduler/pipeline/running', headers=headers,
                          data=json.dumps(body))
        if r.status_code != 200:
            logger.error("startPos:{}, 任务提交失败".format(time))
            exit(-1)
    my_logger.info("任务提交完成")



#!/bin/env python3
# -*- coding: UTF-8 -*-

# Copyright (c) 2020 growingio.com, Inc.  All Rights Reserved

import json
import os

import pandas as pd

from importers.data_import.data_model import EventsCv, EventsJson
from importers.data_import.data_format_util import *
from importers.common import http_util, common_util
from json.decoder import JSONDecodeError
from importers.common.common_util import time_str_to_timestamp_of_tz, getVariables
from importers.common.log_util import logger, my_logger
from importers.common.config_util import BaseConfig
from importers.meta.data_center import getBindEvent, getdataCenterEventVariables, getImportJobStatus, trigger_job

ONE_DAY_MILLISECOND = 86400 * 1000  # 1天(24小时)毫秒值


def events_import(args):
    """
       用户行为导入，按数据格式处理
    """
    # Step one: 校验事件数据基础参数，并预处理
    # 1. 校验时间
    event_start = args.get('event_start')
    if event_start is None:
        logger.error("[-s/--event_start]参数值未指定")
        exit(-1)
    event_end = args.get('event_end')
    if event_end is None:
        logger.error("[-e/--event_end]参数值未指定")
        exit(-1)
    try:
        event_start = time_str_to_timestamp_of_tz(event_start, '%Y-%m-%d', BaseConfig.timezone) * 1000
        event_end = time_str_to_timestamp_of_tz(event_end, '%Y-%m-%d', BaseConfig.timezone) * 1000 + ONE_DAY_MILLISECOND
    except (TypeError, ValueError):
        logger.error("[-s/--event_start]或[-e/--event_end]时间参数格式不对,格式为:YYYY-MM-DD")
        exit(-1)
    # 2. 数据源是否为事件
    ds = args.get('datasource_id')
    if 'HISTORY_EVENT' in ds[1]:
        args['datasource_id'] = ds[1]['HISTORY_EVENT']
    else:
        logger.error("数据源不属于用户行为类型")
        exit(-1)
    # Step one: 按数据格式处理
    f = str(args.get('format'))
    if 'JSON'.__eq__(f):
        events_import_send(
            EventsJson(name=args.get('m'), path=args.get('path'), debug=args.get('debug'), format=f,
                       datasourceId=args.get('datasource_id'), eventStart=event_start, eventEnd=event_end,
                       jobName=args.get('jobName'), clear=args.get('clear'))
        )
    elif 'CSV'.__eq__(f):
        separator = args.get("separator")
        separator = ',' if separator == '' else separator
        events_import_send(
            EventsCv(name=args.get('m'), path=args.get('path'), debug=args.get('debug'), format=f,
                     datasourceId=args.get('datasource_id'), attributes=args.get('attributes'),
                     separator=separator, skipHeader=args.get('skip_header'), eventStart=event_start,
                     eventEnd=event_end, jobName=args.get('jobName'), clear=args.get('clear'))
        )
    elif 'TSV'.__eq__(f):
        separator = args.get("separator")
        separator = '\t' if separator == '' else separator
        events_import_send(
            EventsCv(name=args.get('m'), path=args.get('path'), debug=args.get('debug'), format=f,
                     datasourceId=args.get('datasource_id'), attributes=args.get('attributes'), separator=separator,
                     skipHeader=args.get('skip_header'), eventStart=event_start, eventEnd=event_end,
                     jobName=args.get('jobName'), clear=args.get('clear'))

        )


# SV格式(CSV、TSV)
def sv_import_prepare_process(attributes, paths, skip_header, separator, qualifier, eventStart, eventEnd, debug):
    """
    1. 校验数据基本信息
    2. SV格式数据转换为数据对象
    """
    # Step 1: 校验有无attributes,有无重复列名
    if attributes is None:
        logger.error("[-attr/--attributes]参数值不存在")
        exit(-1)

    commonlist = ['userId', 'event', 'timestamp', 'eventId', 'userKey', 'dataSourceId']
    cols = str(attributes).split(',')
    res = [i for i in cols if i not in commonlist]

    event = getBindEvent()
    cstm_keys = {}
    for i in event['dataCenterCustomEvents']:
        list = []
        for a in i['attributes']:
            list.append(a['key'])
        cstm_keys[i['key']] = list
    # 获取已定义的事件和事件属性
    cstm_attr_keys = getVariables(getdataCenterEventVariables())
    attr_all = BaseConfig.attr_all
    total_error = 0
    count = 0
    for path in paths:
        error = 0
        if skip_header:
            replace_first_line_with_header(path, cols, separator)
        # 读取文件第一行，与预期表头比较
        with open(path, 'r', encoding='utf8') as f:
            first_line = f.readline().strip()
            expected_header = separator.join(cols)
            if first_line != expected_header:
                add_header_to_file(path, cols, separator)

        skh = True
        with open(path, 'r', encoding='utf8') as f:
            for line in f:
                count += 1
                line = line.replace('\n', '').replace('\\t', '\t')
                # Step 2: 校验数据header列是否一致，数量和顺序
                if skh is True and count == 1:
                    header_normal = True
                    error_key = []
                    for key in res:
                        if key not in cstm_attr_keys and key not in attr_all:
                            error_key.append(key)
                    if len(error_key) > 0:
                        logger.error(f"[-attr/--attributes]参数值列:事件属性{error_key}在GIO平台未定义")
                        header_normal = False
                    if not header_normal:
                        logger.error("header校验失败！")
                        return False
                else:
                    # Step 2: 数据列是否一致
                    line_normal = True
                    if check_sv_header_col_count(cols, line.split(separator)) is False:
                        remove_first_line_from_csv(path)
                        logger.error(
                            f"第{count}行:导入文件[{path}]的列数和参数值列数不一致\n请检查文件分隔符，并通过参数[-sep]指定")
                        return False
                    if not line_normal:
                        continue
        if debug:
            df = pd.read_csv(path, sep=separator, header=0 if skh else None,
                             names=cols if skh else None)
            for index, row in df.iterrows():
                count += 1
                col_value = row.to_dict()
                # 转换为数据对象
                attrs = {}
                uid = 'userId' if 'userId' in cols else 'anonymousId'
                for col in cols:
                    if col and col not in ['event', 'userKey', 'timestamp', 'eventId', uid]:
                        if col in cstm_keys.get(col_value['event'], []) or str(col).startswith("$"):
                            attrs[col] = str(col_value[col])
                        else:
                            logger.error(f"不存在事件属性{col}与事件{col_value['event']}绑定关系")
                            return False

                # 提取必要的字段
                userId = col_value.get(uid, '') if uid in col_value else ''
                userKey = col_value.get('userKey', '')
                eventId = col_value.get('eventId')
                dataSourceId = col_value.get('dataSourceId')
                timestamp = col_value.get('timestamp')

                # 校验 userId 和 userKey
                if not userId and (userKey != '$notuser'):
                    logger.error(
                        f"第{index + 1}行: 缺少userId需指定\n若传主体事件,则数据需字段userKey,且值为‘$notuser’")
                    error += 1
                    continue
                # 创建 DataEvent 对象
                data_event = DataEvent(userId=userId, event=col_value['event'], userKey=userKey,
                                       eventId=eventId, timestamp=timestamp, attrs=attrs,
                                       dataSourceId=dataSourceId)

                # 执行数据对象的校验
                is_valid, error_message = validate_data_event(data_event, eventStart, eventEnd, cstm_keys,
                                                              cstm_attr_keys)
                if not is_valid:
                    logger.error(f"第{index + 1}行: 文件[{path}]数据[{row.to_string(index=False)}]:\n{error_message}")
                    error += 1
                    if not skip_header:
                        logger.error(
                            "请检查文件表头数据是否新增以参数[attr]参数为准的表头,请根据信息修改文件内容或参数信息")
                    continue
                if count % 500000 == 0:
                    my_logger.info(f"已经校验{count}条数据......")

            if error > 0:
                logger.warn(f"文件[{path}]导入失败，共发现[{error}]个校验错误")
                total_error += error

    if total_error > 0:
        logger.warn(f"总共发现[{total_error}]个校验错误")
        return False
    else:
        my_logger.info(f"本次共校验[{count}]行数据")
        return True


def events_import_send(events):
    """
       用户行为导入，Json格式数据处理
    """
    # Step 1: 执行Debug
    f = events.format
    if events.debug:
        if 'JSON'.__eq__(f):
            if events_debug_process(events.path, events.eventStart, events.eventEnd) is not True:
                logger.error("Debug校验未通过")
                exit(-1)
        else:
            if sv_import_prepare_process(attributes=events.attributes,
                                         paths=events.path,
                                         skip_header=events.skipHeader,
                                         separator=events.separator,
                                         qualifier=events.qualifier,
                                         eventStart=events.eventStart,
                                         eventEnd=events.eventEnd,
                                         debug=events.debug) is not True:
                logger.error("Debug校验未通过")
                exit(-1)
    else:
        lines_count = count_lines_in_file(events.path)
        my_logger.info(f"本次共导入[{lines_count}]行数据")

    if len(str(events.jobName)) == 0 or events.jobName is None:
        jobName = f"Python_events_{time.strftime('%Y%m%d_%H%M%S', time.localtime(time.time()))}"
    else:
        jobName = events.jobName
    # Step 2: 创建导入任务
    if 'JSON'.__eq__(f):
        job_info = create_task_json(events.eventStart, events.eventEnd, events.datasourceId, jobName)
    else:
        job_info = create_task_sv(events.eventStart, events.eventEnd, events.datasourceId, jobName, events.separator)

    # 任务名重复时，获取不到job信息时，程序直接结束
    if job_info is None:
        logger.error("job_info为空，无法创建导入任务")
        exit(-1)
    else:
        my_logger.info(f"创建导入任务: {job_info}")
    direct = job_info['argument']['directory']
    # Step 3: 上传数据到FTP
    my_logger.info(f"文件开始上传至FTP")
    put_file(events.path, direct)

    # Step 4: 启动导入任务
    start_time = time.time()
    trigger_job(job_info['id'])
    my_logger.info(f"开始执行导入任务")
    flag = True
    while flag:
        eventImportJob = getImportJobStatus(job_info['id'])
        if eventImportJob is not None:
            stage = eventImportJob['stage']
            error = eventImportJob['error']
            if stage is not None and stage.__eq__("FINISH"):
                end_time = time.time()
                cost_time = end_time - start_time
                my_logger.info("导入成功")
                delete_file(events.path, direct)
                my_logger.info("执行导入任务耗时:%.3f秒" % cost_time)
                flag = False
            elif stage is not None and stage.__eq__("ERROR"):
                if error is not None:
                    message = error.get('message', 'No message available')
                else:
                    message = 'Error object is None'
                end_time = time.time()
                cost_time = end_time - start_time
                logger.error(f"导入失败,错误信息为[ {message} ] \n FTP文件路径: {direct}")
                my_logger.info("执行导入任务耗时:%.3f秒" % cost_time)
                exit(-1)
        if flag:
            my_logger.info(f"等待任务完成......")
            time.sleep(10)


def create_task_json(start, end, tid, name):
    """
       创建josn任务
    """
    if len(str(name)) == 0:
        body = '''{ "operationName":"createEventImportJob", "variables":{ "fileType":"ftp", "tunnelId":"%s", 
            "timeRange":"abs:%s,%s", "createType":"PYTHON" }, "query":"mutation createEventImportJob($tunnelId: HashId!, 
            $timeRange: String, $fileType: String, $createType: String) { createEventImportJob(tunnelId: $tunnelId, 
            timeRange: $timeRange, fileType: $fileType, createType: $createType) { id name argument { directory 
            __typename } __typename } }" }''' % (tid, start, end)
    else:
        body = '''{ "operationName":"createEventImportJob", "variables":{ "fileType":"ftp", "tunnelId":"%s", 
            "timeRange":"abs:%s,%s", "name":"%s", "createType":"PYTHON" }, "query":"mutation createEventImportJob($name: 
            String, $tunnelId: HashId!, $timeRange: String, $fileType: String, $createType: String) { 
            createEventImportJob(name: $name, tunnelId: $tunnelId, timeRange: $timeRange, fileType: $fileType, 
            createType: $createType) { id name argument { directory __typename } __typename } }" }''' % (
            tid, start, end, name)

    resp = http_util.send_graphql_post(body.encode('utf-8'))
    try:
        return resp['createEventImportJob']
    except TypeError:
        logger.error("自定义任务名称已存在！")


def create_task_sv(start, end, tid, name, separator):
    """
       创建csv/tsv任务
    """
    separator = separator.replace("\t", "\\\\t")
    if len(str(name)) == 0:
        body = '''{ "operationName":"createEventImportJobV2", "variables":{ "fileType":"csv", "tunnelId":"%s", 
            "timeRange":"abs:%s,%s", "createType":"PYTHON", "csvSeparator":"%s" }, "query":"mutation 
            createEventImportJobV2($tunnelId: HashId!, $timeRange: String, $fileType: String, $createType: String, 
            $csvSeparator: String) { createEventImportJobV2(tunnelId: $tunnelId, timeRange: $timeRange, 
            fileType: $fileType, createType: $createType, csvSeparator: $csvSeparator) { id name argument { directory 
            __typename } __typename } }" }''' % (tid, start, end, separator)
    else:
        body = '''{ "operationName":"createEventImportJobV2", "variables":{ "fileType":"csv", "tunnelId":"%s", 
            "timeRange":"abs:%s,%s", "name":"%s", "createType":"PYTHON", "csvSeparator":"%s" }, "query":"mutation 
            createEventImportJobV2($name: String, $tunnelId: HashId!, $timeRange: String, $fileType: String, 
            $createType: String, $csvSeparator: String) { createEventImportJobV2(name: $name, tunnelId: $tunnelId, 
            timeRange: $timeRange, fileType: $fileType, createType: $createType, csvSeparator: $csvSeparator) { id 
            name argument { directory __typename } __typename } }" }''' % (
            tid, start, end, name, separator)

    resp = http_util.send_graphql_post(body.encode('utf-8'))
    try:
        return resp['createEventImportJobV2']
    except TypeError:
        logger.error("自定义任务名称已存在！")


def events_debug_process(paths, eventStart, eventEnd):
    """
       用户行为导入Debug
       1、校验文件的数据内容合法性, 是否缺失必要字段(userId,event,timestamp)
       2、校验文件的数据时间范围合法性
       3、校验自定义事件在GIO平台是否定义
    """
    bind_event = getBindEvent()
    cstm_keys = {}
    for i in bind_event['dataCenterCustomEvents']:
        list = []
        for a in i['attributes']:
            list.append(a['key'])
        cstm_keys[i['key']] = list
    cstm_attr_keys = getVariables(getdataCenterEventVariables())
    count = 0
    dict_count = 0
    error_count = 0
    correct_count = 0  # 正确行数
    error_dict = {}
    for path in paths:
        my_logger.info(f"start check file: {path}")

        # 1 重命名 文件名 加后缀 _tmp
        names = path.rsplit("/", 1)
        path_tmp = path + "_tmp"
        os.rename(path, path_tmp)  # 如果新文件名的文件 存在 则覆盖 ok的

        names2 = names[0].rsplit("/", 1)
        grandpa_path = names2[0]
        father_path = names2[-1]
        error_file_name = names[-1] + '_' + str(int(round(time.time() * 1000))) + "_error"

        # 错误空文件名
        dir_error = f"{grandpa_path}/{father_path}_error"
        path_error = f"{dir_error}/{error_file_name}"
        if not os.path.exists(dir_error):
            os.makedirs(dir_error)

        # 2 打开 每个文件， 加后缀 _tmp 文件
        with open(path_tmp, 'r', encoding='utf8') as f:

            # 打开正确空文件
            with open(path, 'w', encoding='utf8') as f_correct:

                # 打开错误空文件
                with open(path_error, 'w', encoding='utf8') as f_error:
                    current_file_error_lines = 0
                    for line in f:
                        count = count + 1
                        source_line = line  # 把原始行数据 保存到 source_line 变量中，后面判断 正常 异常 ，再写入到相应的文件中
                        line = line.replace('\n', '')
                        if not line == '':
                            try:
                                json_data = json.loads(line.strip())
                            except JSONDecodeError:
                                logger.error(f"第{count}行:文件[{path}]数据[{line}]:\n数据非JSON格式\n")
                                error_count += 1
                                current_file_error_lines += 1
                                f_error.write(source_line)  # 写入异常文件
                                continue

                            data_event, message = extract_and_validate_data(json_data)
                            if data_event is None:
                                logger.error(f"第{count}行:文件[{path}]数据[{line}]\n"
                                             f"{message}")
                                error_count += 1
                                current_file_error_lines += 1
                                f_error.write(source_line)  # 写入异常文件
                                continue

                            # 调用公共校验方法进行校验
                            is_valid, error_message = validate_data_event(data_event, eventStart, eventEnd, cstm_keys,
                                                                          cstm_attr_keys)
                            if not is_valid:  # 异常
                                error_hash = hash(error_message)
                                if error_hash not in error_dict:
                                    error_dict[error_hash] = error_message
                                    logger.error(f"第{count}行:文件[{path}]数据[{line}]:\n"
                                                 f"{error_message}")
                                    dict_count += 1
                                error_count += 1
                                current_file_error_lines += 1
                                f_error.write(source_line)  # 写入异常文件
                            else:  # 正常
                                f_correct.write(source_line)  # 写入正常文件
                                correct_count = correct_count + 1
                            if count % 500000 == 0:
                                my_logger.info(f"已经校验{count}条数据......")

                            if dict_count >= 10000:
                                logger.error(
                                    f"数据内容错误条数已经超过 10000 条, 请先修正数据, 异常数据已剪切到如下目录中 {dir_error}")
                                exit(-1)
                        else:
                            logger.warn(f"第{count}行为空，跳过该行")
                f_error.close()  # 关闭 异常文件
            f_correct.close()  # 关闭 正常文件
        f.close()  # 关闭 带有后缀 _tmp的文件

        # 删除 带有后缀 _tmp的文件
        os.remove(path_tmp)
        # 判断 若 异常文件空白 行数=0，则 删除 异常文件
        if current_file_error_lines == 0:
            os.remove(path_error)

    # 判断 若 异常 文件夹 空白，则 删除 该文件夹
    if len(os.listdir(dir_error)) == 0:
        os.removedirs(dir_error)

    if error_count == 0:
        my_logger.info(f"本次共校验[{count}]行数据")
    else:
        my_logger.info(
            f"本次共校验[{count}]行数据,其中校验失败[{error_count}]行数据, 包含[{dict_count}]种错误类型，具体错误内容看上方日志详情,异常数据已剪切到如下目录中 {dir_error}")

    if correct_count == 0:
        my_logger.info(f"由于本次正确数据0条，故不生成导数任务。")
        return False
    else:
        return True



import csv
import json
import os

import pandas as pd

from importers.common import http_util
from importers.common.config_util import ApiConfig, get_temp_dir_from_config
from importers.data_import.data_model import ItemVariablesJson, ItemVariablesSv
from importers.data_import.data_format_util import *
from importers.common.log_util import logger, my_logger
from importers.meta.check_util import check_item_var_key_data_exsit, check_item_var_exsit
from importers.meta.data_center import trigger_job, getImportJobStatus


def item_variables_import(args):
    """
     主体导入，按数据格式处理
    """
    ds = args.get('datasource_id')
    if 'HISTORY_ITEM' in ds[1]:
        args['datasource_id'] = ds[1]['HISTORY_ITEM']
    else:
        logger.error("数据源不属于主体数据类型")
        exit(-1)
    # Step one: 按数据格式处理
    f = str(args.get('format'))
    if 'JSON'.__eq__(f):
        item_variables_import_send(
            ItemVariablesJson(name=args.get('m'), path=args.get('path'), debug=args.get('debug'), format=f,
                              datasourceId=args.get('datasource_id'), itemKey=args.get('item_key'),
                              jobName=args.get('jobName'),
                              clear=args.get('clear'))
        )
    elif 'CSV'.__eq__(f):
        separator = args.get("separator")
        separator = ',' if separator == '' else separator
        item_variables_import_send(
            ItemVariablesSv(name=args.get('m'), path=args.get('path'), debug=args.get('debug'), format=f,
                            datasourceId=args.get('datasource_id'), itemKey=args.get('item_key'),
                            jobName=args.get('jobName'),
                            attributes=args.get('attributes'), separator=separator, skipHeader=args.get('skip_header'),
                            clear=args.get('clear'))
        )
    elif 'TSV'.__eq__(f):
        separator = args.get("separator")
        separator = '\t' if separator == '' else separator
        item_variables_import_send(
            ItemVariablesSv(name=args.get('m'), path=args.get('path'), debug=args.get('debug'), format=f,
                            datasourceId=args.get('datasource_id'), itemKey=args.get('item_key'),
                            jobName=args.get('jobName'),
                            attributes=args.get('attributes'), separator=separator, skipHeader=args.get('skip_header'),
                            clear=args.get('clear'))
        )


def sv_import_prepare_process(attributes, paths, skip_header, separator, qualifier, itemKey):
    """
      1.校验数据基本信息
      2.CSV/TSV格式数据处理
    """
    # Step 1: 校验有无attributes,有无重复列名
    if attributes is None:
        logger.error(f"[-attr/--attributes]参数值不存在")
        exit(-1)

    cols = str(attributes).split(',')
    duplicate_col = check_sv_col_duplicate(cols)
    flag, var_id, var_name, var_desc = check_item_var_exsit(itemKey)
    if flag:
        keys = check_item_var_key_data_exsit(var_id)
    else:
        logger.error(f"item_Key主体标识符[{itemKey}]不存在，校验终止")
        exit(-1)

    if duplicate_col is not None:
        logger.error(f"[-attr/--attributes]出现重复列值[{duplicate_col}]")
        exit(-1)
    count = 0
    for path in paths:
        if skip_header:
            replace_first_line_with_header(path, cols, separator)
        # 读取文件第一行，与预期表头比较
        with open(path, 'r', encoding='utf8') as f:
            first_line = f.readline().strip()
            expected_header = separator.join(cols)
            if first_line != expected_header:
                add_header_to_file(path, cols, separator)

        skip_header = True
        with open(path, 'r', encoding='utf8') as f:
            for line in f:
                count += 1
                # Step 2: 数据列是否一致
                line = line.replace('\n', '').replace('\\t', '\t')
                line_normal = True
                if check_sv_header_col_count(cols, line.split(separator)) is False:
                    if not check_csv_header_col_count(cols, line, separator):
                        remove_first_line_from_csv(path)
                        logger.error(
                            f"第{count}行:导入文件[{path}]的列数和参数值列数不一致\n请检查文件分隔符，并通过参数[-sep]指定")
                        return False
                if not line_normal:
                    continue
        df = pd.read_csv(path, sep=separator, header=0 if skip_header else None,
                         names=cols if not skip_header else None)
        # Step 2 & 3: 校验数据列是否一致
        for index, row in df.iterrows():
            if len(row) != len(cols):
                remove_first_line_from_csv(path)
                logger.error(
                    f"文件[{path}]的第{index + 1}行数据列数与文件头部列数不一致\n请检查文件分隔符，并通过参数[-sep]指定")
                exit(-1)

            col_value = row.to_dict()
            attrs = {key: value for key, value in col_value.items() if
                     len(str(value)) != 0 and key != 'item_id' and not key.startswith('$')}

            # 确保 attrs 中的所有键都在 keys 中
            if not all(key in keys for key in attrs):
                invalid_keys = [key for key in attrs if key not in keys]
                logger.error(f"以下列名主体字段中不存在: {invalid_keys}")
                exit(-1)
    my_logger.info(f"本次共校验[{count}]行数据")
    return True


def item_variables_import_send(itemVariables):
    """
       主体，Json格式数据处理
    """
    itemKey = itemVariables.itemKey
    f = itemVariables.format
    # Step 1: 执行Debug
    if itemVariables.debug:
        if 'JSON'.__eq__(f):
            if json_variables_debug_process(itemVariables.path, itemKey) is not True:
                logger.error("Debug校验未通过")
                exit(-1)
        else:
            if sv_import_prepare_process(attributes=itemVariables.attributes,
                                         paths=itemVariables.path,
                                         skip_header=itemVariables.skipHeader,
                                         separator=itemVariables.separator,
                                         qualifier=itemVariables.qualifier,
                                         itemKey=itemKey) is not True:
                logger.error("Debug校验未通过")
                exit(-1)
    else:
        lines_count = count_lines_in_file(itemVariables.path)
        my_logger.info(f"本次共导入[{lines_count}]行数据")

    if len(str(itemVariables.jobName)) == 0 or itemVariables.jobName is None:
        jobName = f"Python_item_{time.strftime('%Y%m%d_%H%M%S', time.localtime(time.time()))}"
    else:
        jobName = itemVariables.jobName

    # Step 2: 创建导入任务
    if 'JSON'.__eq__(f):
        job_info = create_task_json(itemVariables.datasourceId, jobName, itemKey)
    else:
        job_info = create_task_sv(itemVariables.datasourceId, jobName, itemKey, itemVariables.separator)

    # 任务名重复时，获取不到job信息时，程序直接结束
    if job_info is None:
        logger.error("job_info为空，无法创建导入任务")
        exit(-1)
    else:
        my_logger.info(f"创建导入任务: {job_info}")
    direct = job_info['argument']['directory']
    # Step 3: 上传数据到FTP
    my_logger.info(f"文件开始上传至FTP")
    put_file(itemVariables.path, direct)

    # Step 4: 启动导入任务
    start_time = time.time()
    trigger_job(job_info['id'])
    my_logger.info(f"开始执行导入任务")
    flag = True
    while flag:
        eventImportJob = getImportJobStatus(job_info['id'])
        if eventImportJob is not None:
            stage = eventImportJob['stage']
            error = eventImportJob['error']
            if stage is not None and stage.__eq__("FINISH"):
                end_time = time.time()
                cost_time = end_time - start_time
                my_logger.info("导入成功")
                delete_file(itemVariables.path, direct)
                my_logger.info("执行导入任务耗时:%.3f秒" % cost_time)
                flag = False
            elif stage is not None and stage.__eq__("ERROR"):
                end_time = time.time()
                cost_time = end_time - start_time
                if error is not None:
                    message = error.get('message', 'No message available')
                else:
                    message = 'Error object is None'
                logger.error(f"导入失败,错误信息为[ {message} ] \n FTP文件路径: {direct}")
                my_logger.info("执行导入任务耗时:%.3f秒" % cost_time)
                exit(-1)
        if flag:
            my_logger.info(f"等待任务完成......")
            time.sleep(10)


def create_task_json(ds, name, itemKey):
    """
           创建任务,允许用户自定义更改任务名称
        """
    if len(str(name)) == 0:
        body = '''{ "operationName": "createEventImportJob", "variables": { "fileType":"ftp", "timeRange":"", 
        "tunnelId": "%s", "itemModel":{ "key": "%s" }, "createType":"PYTHON" }, "query": "mutation 
        createEventImportJob($tunnelId: HashId!, $timeRange: String, $fileType: String, $itemModel: ItemModelIdInput, 
        $createType: String) { createEventImportJob(tunnelId: $tunnelId, timeRange: $timeRange,fileType: $fileType, 
        itemModel:$itemModel, createType: $createType) { id name argument { directory __typename } __typename } }" 
        }''' % (ds, itemKey)
    else:
        body = '''{ "operationName":"createEventImportJob", "variables":{ "fileType":"ftp", "tunnelId":"%s", 
            "timeRange":"", "name":"%s", "itemModel":{ "key": "%s" }, "createType":"PYTHON" }, "query":"mutation 
            createEventImportJob($name: String, $tunnelId: HashId!, $timeRange: String, $fileType: String, $itemModel: 
            ItemModelIdInput, $createType: String) { createEventImportJob(name: $name, tunnelId: $tunnelId, timeRange: 
            $timeRange, fileType: $fileType, itemModel:$itemModel, createType: $createType) { id name argument { 
            directory __typename } __typename } }" }''' % (ds, name, itemKey)
    resp = http_util.send_graphql_post(body.encode('utf-8'))
    try:
        return resp['createEventImportJob']
    except TypeError:
        logger.error("自定义任务名称已存在！")


def create_task_sv(ds, name, itemKey, separator):
    """
           创建任务,允许用户自定义更改任务名称
        """
    separator = separator.replace("\t", "\\\\t")
    if len(str(name)) == 0:
        body = '''{ "operationName": "createEventImportJobV2", "variables": { "fileType":"csv", "timeRange":"", 
        "tunnelId": "%s", "itemModel":{ "key": "%s" }, "createType":"PYTHON", "csvSeparator":"%s" }, 
        "query": "mutation createEventImportJobV2($tunnelId: HashId!, $timeRange: String, $fileType: String, 
        $itemModel: ItemModelIdInput, $createType: String, $csvSeparator: String) { createEventImportJobV2(tunnelId: 
        $tunnelId, timeRange: $timeRange,fileType: $fileType, itemModel:$itemModel, createType: $createType, 
        csvSeparator: $csvSeparator) { id name argument { directory __typename } __typename } }" }''' % (
            ds, itemKey, separator)
    else:
        body = '''{ "operationName":"createEventImportJobV2", "variables":{ "fileType":"csv", "tunnelId":"%s", 
        "timeRange":"", "name":"%s", "itemModel":{ "key": "%s" }, "createType":"PYTHON", "csvSeparator":"%s" }, 
        "query":"mutation createEventImportJobV2($name: String, $tunnelId: HashId!, $timeRange: String, $fileType: 
        String, $itemModel: ItemModelIdInput, $createType: String, $csvSeparator: String) { createEventImportJobV2(
        name: $name, tunnelId: $tunnelId, timeRange: $timeRange, fileType: $fileType, itemModel:$itemModel, 
        createType: $createType, csvSeparator: $csvSeparator) { id name argument { directory __typename } __typename 
        } }" }''' % (ds, name, itemKey, separator)
    resp = http_util.send_graphql_post(body.encode('utf-8'))
    try:
        return resp['createEventImportJobV2']
    except TypeError:
        logger.error("自定义任务名称已存在！")


def json_variables_debug_process(paths, itemKey):
    """
    主体导入Debug
    1、校验有无itemKey
    2、校验主体(条件:是否是平台内置和是否定义)
    """
    count = 0
    dict_count = 0
    error_count = 0
    correct_count = 0  # 正确行数
    error_dict = {}
    flag, var_id, var_name, var_desc = check_item_var_exsit(itemKey)
    if flag:
        key_list = check_item_var_key_data_exsit(var_id)
    else:
        logger.error(f"item_Key主体标识符[{itemKey}]不存在，校验终止")
        return False  # 直接退出校验并返回 False
    temp_dir = get_temp_dir_from_config()  # 从配置中获取临时存储目录
    current_tmp_name = str(int(round(time.time() * 1000))) + "_error"
    current_tmp_path = os.path.join(temp_dir, current_tmp_name)
    if not os.path.exists(current_tmp_path):
        os.makedirs(current_tmp_path)
    for path in paths:
        # Define error file path
        error_file_name = 'item_' + str(int(round(time.time() * 1000))) + '_error.json'
        error_path = current_tmp_path + '/' + error_file_name

        with open(path, 'r', encoding='utf8') as f:
            with open(error_path, 'w', encoding='utf8') as f_error:
                lines_to_write = []  # Collect lines to write back to original file
                for line in f:
                    count = count + 1
                    line = line.replace('\n', '')
                    if not line == '':
                        normal = True
                        error_message = ""
                        try:
                            data_dictionary = json.loads(line)
                            # item_id
                            if 'item_id' not in data_dictionary:
                                normal = False
                                error_message += f"item_id不存在\n"
                            # 主体
                            if 'attrs' in data_dictionary:
                                if not isinstance(data_dictionary['attrs'], dict):
                                    normal = False
                                    error_message += f"attrs数据格式不对\n"
                                for key in data_dictionary['attrs']:
                                    if key not in key_list:
                                        normal = False
                                        error_message += f"主体字段[{key}]不存在\n"
                                    elif data_dictionary['attrs'][key] is None or data_dictionary['attrs'][key] == "":
                                        print(
                                            f"主体[{data_dictionary['item_id']}]中字段[{key}]的值为空或为NULL,请检查原始数据\n")
                        except json.JSONDecodeError:
                            normal = False
                            error_message += f"文件[{path}]数据[{line}]非JSON格式\n"

                        if not normal:  # 异常
                            error_hash = hash(error_message)
                            if error_hash not in error_dict:
                                error_dict[error_hash] = error_message
                                logger.error(f"第{count}行:文件[{path}]数据[{line}],\n"
                                             f"{error_message}")
                                dict_count += 1
                            error_count += 1
                            f_error.write(line + '\n')  # 写入异常数据到错误文件
                        else:  # 正常
                            lines_to_write.append(line)  # 添加到待写入的行列表
                            correct_count = correct_count + 1
                        if count % 500000 == 0:
                            my_logger.info(f"已经校验{count}条数据......")

                        if dict_count >= 10000:
                            logger.error("数据内容错误条数已经超过 10000 条, 请先修正数据 ")
                            exit(-1)
                    else:
                        logger.warn(f"第{count}行为空，跳过该行")

                # Write valid lines back to the original file
                with open(path, 'w', encoding='utf8') as f_original:
                    for line in lines_to_write:
                        f_original.write(line + '\n')

        f_error.close()  # 关闭错误文件
        f.close()  # 关闭原始文件
        # 判断 若 异常文件空白 行数=0，则 删除 异常文件
        if error_count == 0:
            os.remove(error_path)
    if len(os.listdir(current_tmp_path)) == 0:
        os.removedirs(current_tmp_path)

    if error_count == 0:
        my_logger.info(f"本次共校验[{count}]行数据")
    else:
        my_logger.info(
            f"本次共校验[{count}]行数据,其中校验失败[{error_count}]行数据,包含[{dict_count}]种错误类型，具体错误内容看上方日志详情,异常数据已剪切到临时文件目录[{current_tmp_path}]")

    if correct_count == 0:
        my_logger.info(f"由于本次正确数据0条，故不生成导数任务。")
        return False
    else:
        return True
